package jcg.zheng.demo;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.Collections;
import java.util.Map.Entry;
import java.util.SortedMap;
import java.util.TreeMap;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import jcg.zheng.demo.data.User;
import jcg.zheng.demo.data.UserSalaryComparator;
import jcg.zheng.demo.treemap.PrintService;

public class TreeMapTest {

	private static final String DEFAULT_ASCENDING_ORDER_OF_KEY = "** Default ascending order based on the Key value **";

	private TreeMap<Integer, String> intTreeMapWithDefaultOrder;
	private TreeMap<Integer, String> intTreeMapWithReverseOrder;
	private TreeMap<User, String> userTreeMapWithDefaultOrder;
	private TreeMap<User, String> userTreeMapOrderBySalary;

	private MapTestUtil testUtil = new MapTestUtil();
	private PrintService mapService = new PrintService();

	@Before
	public void setup() {
		intTreeMapWithDefaultOrder = new TreeMap<>();
		intTreeMapWithReverseOrder = new TreeMap<>(Collections.reverseOrder());

		userTreeMapWithDefaultOrder = new TreeMap<>();
		userTreeMapOrderBySalary = new TreeMap<>(new UserSalaryComparator());
	}

	@After
	public void test_isEmpty() {
		intTreeMapWithDefaultOrder.clear();
		intTreeMapWithReverseOrder.clear();
		userTreeMapWithDefaultOrder.clear();
		userTreeMapOrderBySalary.clear();
		
		assertTrue(intTreeMapWithDefaultOrder.isEmpty());
		assertTrue(intTreeMapWithReverseOrder.isEmpty());
		assertTrue(userTreeMapWithDefaultOrder.isEmpty());
		assertTrue(userTreeMapOrderBySalary.isEmpty());
	}

	@Test
	public void test_integer_key() {
		testUtil.add4Elements(intTreeMapWithDefaultOrder);
		System.out.println(DEFAULT_ASCENDING_ORDER_OF_KEY);
		mapService.printIntegerTreeMap_forEach(intTreeMapWithDefaultOrder);

		Entry<Integer, String> firstEntry = intTreeMapWithDefaultOrder.firstEntry();
		System.out.println("* firstEntry = " + firstEntry.toString());

		Integer firstKey = intTreeMapWithDefaultOrder.firstKey();
		System.out.println(" firstKey=" + firstKey.intValue());

		Entry<Integer, String> lastEntry = intTreeMapWithDefaultOrder.lastEntry();
		System.out.println("* lastEntry.key= " + lastEntry.getKey().intValue());

		// will only print out {3=XXX} as it the only element whose key value < 10
		SortedMap<Integer, String> headMap = intTreeMapWithDefaultOrder.headMap(10);
		System.out.println("** headMap key < 10 in default order ***");
		mapService.printIntegerTreeMap_forEach(headMap);

		// will only print out {11=ABC} as it the only element whose key value > 3
		SortedMap<Integer, String> tailMap = intTreeMapWithDefaultOrder.tailMap(10);
		System.out.println("** tailMap key > 10 in default order ***");
		mapService.printIntegerTreeMap_forEach(tailMap);

		Entry<Integer, String> firstPulled = intTreeMapWithDefaultOrder.pollFirstEntry();
		System.out.println(" firstPulled= " + firstPulled.toString());
		assertEquals(3, intTreeMapWithDefaultOrder.size());

		mapService.print(intTreeMapWithDefaultOrder);
		mapService.printIntergerTreeMap_values(intTreeMapWithDefaultOrder);
		
		testUtil.removeElements(intTreeMapWithDefaultOrder);
	
	}

	@Test(expected = NullPointerException.class)
	public void will_throw_NullPointerException() {
		intTreeMapWithDefaultOrder.put(null, "test");
	}

	@Test
	public void test_integer_key_ReversOrder() {
		assertTrue(intTreeMapWithReverseOrder.isEmpty());
		testUtil.add4Elements(intTreeMapWithReverseOrder);
		System.out.println(" *** integerTreeMapWithReverseOrder in ReverseOrder ** ");
		mapService.printIntegerTreeMap_forEach(intTreeMapWithReverseOrder);

		TreeMap<Integer, String> createdFromSortedMap = new TreeMap<>(intTreeMapWithReverseOrder);
		System.out.println(" *** createdFromSortedMap in ReverseOrder ** ");
		mapService.printIntegerTreeMap_forEach(createdFromSortedMap);
	}

	@Test
	public void test_Key_Comparable() {
		assertTrue(userTreeMapWithDefaultOrder.isEmpty());
		testUtil.add4Users(userTreeMapWithDefaultOrder);

		System.out.println(" *** Order based on User's compareTo() ***");
		mapService.printUserTreeMap_entrySet(userTreeMapWithDefaultOrder);
		
		TreeMap<User, String> createdFromSortedMap = new TreeMap<>(userTreeMapWithDefaultOrder);
		assertEquals(userTreeMapWithDefaultOrder.size(), createdFromSortedMap.size());
		System.out.println("***** createdFromSortedMap order from natual order *****");
		mapService.printUserTreeMap_entrySet(createdFromSortedMap);
	}

	@Test
	public void test_Key_Comparator() {
		assertTrue(userTreeMapOrderBySalary.isEmpty());
		testUtil.add4Users(userTreeMapOrderBySalary);

		System.out.println(" *** Ordered based on User's Salary() ***");
		mapService.printUserTreeMap_entrySet(userTreeMapOrderBySalary);

		TreeMap<User, String> createdFromSortedMap = new TreeMap<>(userTreeMapOrderBySalary);
		assertEquals(userTreeMapOrderBySalary.size(), createdFromSortedMap.size());
		System.out.println("***** createdFromSortedMap order by Salary *****");
		mapService.printUserTreeMap_entrySet(createdFromSortedMap);
		assertTrue(createdFromSortedMap.equals(userTreeMapOrderBySalary));

		createdFromSortedMap.put(new User("JCG", "mZheng", 11), "My Name5");
		assertFalse(createdFromSortedMap.equals(userTreeMapOrderBySalary));

		Entry<User, String> lastEntry = createdFromSortedMap.lastEntry();
		System.out.println("LastEntry = " + lastEntry.toString());

		Entry<User, String> ceilingEntry = createdFromSortedMap.ceilingEntry(lastEntry.getKey());
		System.out.println("ceilingEntry = " + ceilingEntry.toString());
	}

}
